#!/usr/bin/perl -w 
############################## check_snmp_syno_diskhealth.pl ########
my $Version='1.0';
# Date : Mar 14 2017
# Author  : Faes Nathan
# Help : esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;

my $PROGNAME='check_snmp_fortinet.pl';
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my $return=0;
my ($opt_h,$opt_V);
my $community = "public";
my $hostname;
my $snmp_version = 2;
my $warning = 80;
my $critical = 90;
my $type = "CPU";
## OIDs ##
my $oid_unitdesc         = ".1.3.6.1.2.1.1.5.0";                   # Location of Fortinet device description... (String)
my $oid_serial           = ".1.3.6.1.4.1.12356.100.1.1.1.0";       # Location of Fortinet serial number (String)
my $oid_firmware         = ".1.3.6.1.4.1.12356.101.4.1.1.0";       # Location of Fortinet firmware
my $oid_cpu              = ".1.3.6.1.4.1.12356.101.13.2.1.1.3";    # Location of cluster member CPU (%)
my $oid_net              = ".1.3.6.1.4.1.12356.101.13.2.1.1.5";    # Location of cluster member Net (kbps)
my $oid_mem              = ".1.3.6.1.4.1.12356.101.13.2.1.1.4";    # Location of cluster member Mem (%)
my $oid_ses              = ".1.3.6.1.4.1.12356.101.13.2.1.1.6";    # Location of cluster member Sessions (int)

## FortiAnalyzer OIDs ##
my $oid_faz_cpu_used     = ".1.3.6.1.4.1.12356.103.2.1.1.0";       # Location of CPU for FortiAnalyzer (%)
my $oid_faz_mem_used     = ".1.3.6.1.4.1.12356.103.2.1.2.0";       # Location of Memory used for FortiAnalyzer (kb)
my $oid_faz_mem_avail    = ".1.3.6.1.4.1.12356.103.2.1.3.0";       # Location of Memory available for FortiAnalyzer (kb)
my $oid_faz_disk_used    = ".1.3.6.1.4.1.12356.103.2.1.4.0";       # Location of Disk used for FortiAnalyzer (Mb)
my $oid_faz_disk_avail   = ".1.3.6.1.4.1.12356.103.2.1.5.0";       # Location of Disk available for FortiAnalyzer (Mb)

## FortiMail OIDs ##
my $oid_fe_cpu           = ".1.3.6.1.4.1.12356.105.1.6.0";         # Location of CPU for FortiMail (%)
my $oid_fe_mem           = ".1.3.6.1.4.1.12356.105.1.7.0";         # Location of Memory used for FortiMail (%)
my $oid_fe_ldisk         = ".1.3.6.1.4.1.12356.105.1.8.0";         # Location of Log Disk used for FortiMail (%)
my $oid_fe_mdisk         = ".1.3.6.1.4.1.12356.105.1.9.0";         # Location of Mail Disk used for FortiMail (%)
my $oid_fe_load          = ".1.3.6.1.4.1.12356.105.1.30.0";        # Location of Load used for FortiMail (%)
my $oid_fe_ses           = ".1.3.6.1.4.1.12356.105.1.10.0";        # Location of cluster member Sessions for FortiMail (int)

# Cluster
my $oid_cluster_type     = ".1.3.6.1.4.1.12356.101.13.1.1.0";      # Location of Fortinet cluster type (String)
my $oid_cluster_serials  = ".1.3.6.1.4.1.12356.101.13.2.1.1.2";    # Location of Cluster serials (String)
my $oid_cluster_sync_state = ".1.3.6.1.4.1.12356.101.13.2.1.1.12"; # Location of cluster sync state (int)

# VPN OIDs
# XXX to be checked
my $oid_ActiveSSL         = ".1.3.6.1.4.1.12356.101.12.2.3.1.2.1"; # Location of Fortinet firewall SSL VPN Tunnel connection count
my $oid_ActiveSSLTunnel   = ".1.3.6.1.4.1.12356.101.12.2.3.1.6.1"; # Location of Fortinet firewall SSL VPN Tunnel connection count
my $oid_ipsectuntableroot = ".1.3.6.1.4.1.12356.101.12.2.2.1";     # Table of IPSec VPN tunnels
my $oid_VPNtunStatus        = ".1.3.6.1.4.1.12356.101.12.2.2.1.20";                                 # Location of a tunnel's connection status
my $oid_VPNtunIndex       = ".1.3.6.1.4.1.12356.101.12.2.2.1.1";                                  # Location of a tunnel's index...
my $oidf_tunname          = ".3";                                  # Location of a tunnel's name...

# WTP
my $oid_apstatetableroot  = ".1.3.6.1.4.1.12356.101.14.4.4.1.7";   # Represents the connection state of a WTP to AC : offLine(1), onLine(2), downloadingImage(3), connectedImage(4), other(0)
my $oid_wtpsessions       = ".1.3.6.1.4.1.12356.101.14.2.5.0";     # Represents the number of WTPs that are connecting to the AC.
my $oid_wtpmanaged        = ".1.3.6.1.4.1.12356.101.14.2.4.0";     # Represents the number of WTPs being managed on the AC
my $oid_apipaddrtableroot = ".1.3.6.1.4.1.12356.101.14.4.4.1.3" ;  # Represents the IP address of a WTP
my $oid_apidtableroot     = ".1.3.6.1.4.1.12356.101.14.4.4.1.1" ;  # Represents the unique identifier of a WTP

# HARDWARE SENSORS
# "A list of device specific hardware sensors and values. Because different devices have different hardware sensor capabilities, this table may or may not contain any values."
my $oid_hwsensor_cnt     = ".1.3.6.1.4.1.12356.101.4.3.1.0";       # Hardware Sensor count
my $oid_hwsensorid       = ".1.3.6.1.4.1.12356.101.4.3.2.1.1";     # Hardware Sensor index
my $oid_hwsensorname     = ".1.3.6.1.4.1.12356.101.4.3.2.1.2";     # Hardware Sensor Name
my $oid_hwsensorvalue    = ".1.3.6.1.4.1.12356.101.4.3.2.1.3";     # Hardware Sensor Value
my $oid_hwsensoralarm    = ".1.3.6.1.4.1.12356.101.4.3.2.1.4";     # Hardware Sensor Alarm (not all sensors have alarms!)


sub print_help() {
	printf "check la charge CPU des Fortinet.\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)     Hostname to query - (required)\n";
	print " -C (--community)    SNMP read community (defaults to public)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "                     2 for SNMP v2c (default)\n";
	print " -w                  warning disk temperature (default: 40)\n";
	print " -c                  critical disk temperature (default: 50)\n";
	print " -V (--version)      Plugin version\n";
	print " -h (--help)         usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>] [-w warning] [-c critical]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}
my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucle si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
} 
#
#MAIN
#
&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"C=s" => \$community,	"community=s" => \$community,
	"H=s" => \$hostname,	"hostname=s" => \$hostname,
	"v=i" => \$snmp_version,"snmp_version=i" => \$snmp_version,
	"T=s" => \$type,"type=s" => \$type,
	"w=i" => \$warning,"warning=i" => \$warning,
	"c=i" => \$critical,"critical=i" => \$critical,
);
# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }
if (!$hostname) { print_help(); exit $ERRORS{'UNKNOWN'}; }
my ($snmp, $snmperror);
if ($snmp_version == 2) 
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv2c',
		-community => $community,
		-timeout=>2
	);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv1',
		-community => $community,
		-timeout=>2
	);
}
#on interroge en SNMP pour récupérer les informations 
my $result = $snmp->get_request("$oid_serial");

#si l'host ne répond pas on blocke le script
if (!defined($result)) 
{
	if($snmp_version == 2)
	{
		($snmp, $snmperror) = Net::SNMP->session(
			-hostname => $hostname,
			-version => 'snmpv1',
			-community => $community
		);
		$result = $snmp->get_request("$oid_serial");
		if (!defined($result)) 
		{
	   		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
	   		$snmp->close;
	  		 exit $ERRORS{"UNKNOWN"};
		}
	}
	else
	{
		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
		$snmp->close;
		exit $ERRORS{"UNKNOWN"};
	}
}
my $curr_serial = $$result{$oid_serial};	
$result = $snmp->get_request("$oid_unitdesc");
my $curr_desc = $$result{$oid_unitdesc};	
my $message ="";
my $return_state = "OK";

if($type eq 'CPU' || $type eq 'cpu')
{
	$message = get_cpu();
}
elsif($type eq 'MEM'|| $type eq 'mem')
{
	$message = get_mem();
}
elsif($type eq 'Ses')
{
	$message = get_session();
}
elsif($type eq 'VPN')
{
	$message = get_vpn_state();
}
print "$message";
exit($return);

sub get_cpu{
	my $message="";
	my $cpu = $snmp->get_table(Baseoid => $oid_cpu);
	if (!defined($cpu)) 
	{
		print "CPU na\n";
		$return = priority_state($return,$ERRORS{UNKNOWN});
	}
	else
	{
		my $nbrcpu = 0;
		my $cpu_val = 0;
		my $status = "OK";
		foreach my $key ( keys %$cpu) 
		{	
			if($key =~/\.([0-9]*)$/)
			{
				my $id = $1;
				$cpu_val = $cpu_val + $$cpu{$key};		
				$nbrcpu++;
			}
		}
		my $endline="< ".$warning."%;";
		$cpu_val = $cpu_val / $nbrcpu; 
		if ( $cpu_val >= $critical ) 
		{
			$status = "CRITICAL";
			$return = priority_state($return,$ERRORS{CRITICAL});
			$endline="> ".$critical."%;";
		} 
		elsif ( $cpu_val >= $warning ) 
		{
			$status = "WARNING";
			$return = priority_state($return,$ERRORS{WARNING});
			$endline="> ".$warning."%;";
		} 
		else 
		{
			$status = "OK";
			$return = priority_state($return,$ERRORS{OK});
		}
		$message = "$status : $curr_desc(Current device: $curr_serial) $nbrcpu CPU, load ".$cpu_val."% $endline |'cpu'=".$cpu_val."%;".$warning.";".$critical."\n";

	}
	return $message;
}
sub get_mem{
	my $message="";
	my $mem = $snmp->get_table(Baseoid => $oid_mem);
	if (!defined($mem)) 
	{
		print "Memory na\n";
		$return = priority_state($return,$ERRORS{UNKNOWN});
	}
	else
	{
		my $mem_val = 0;
		my $status = "OK";
		foreach my $key ( keys %$mem) 
		{	
			$mem_val = $$mem{$key};
		}
		my $endline="< ".$warning."%;";
		if ( $mem_val >= $critical ) 
		{
			$status = "CRITICAL";
			$return = priority_state($return,$ERRORS{CRITICAL});
			$endline="> ".$critical."%;";
		} 
		elsif ( $mem_val >= $warning ) 
		{
			$status = "WARNING";
			$return = priority_state($return,$ERRORS{WARNING});
			$endline="> ".$warning."%;";
		} 
		else 
		{
			$status = "OK";
			$return = priority_state($return,$ERRORS{OK});
		}
		$message = "$status : $curr_desc(Current device: $curr_serial) memory is ".$mem_val."% $endline |'memory'=".$mem_val."%;".$warning.";".$critical."\n";

	}
	return $message;
}
sub get_session{
	my $message="";
	my $ses = $snmp->get_table(Baseoid => $oid_ses);
	if (!defined($ses)) 
	{
		print "Session na\n";
		$return = priority_state($return,$ERRORS{UNKNOWN});
	}
	else
	{
		my $ses_val = 0;
		my $status = "OK";
		foreach my $key ( keys %$ses) 
		{	
			$ses_val = $$ses{$key};
		}
		my $endline="< ".$warning.";";
		if ( $ses_val >= $critical ) 
		{
			$status = "CRITICAL";
			$return = priority_state($return,$ERRORS{CRITICAL});
			$endline="> ".$critical.";";
		} 
		elsif ( $ses_val >= $warning ) 
		{
			$status = "WARNING";
			$return = priority_state($return,$ERRORS{WARNING});
			$endline="> ".$warning.";";
		} 
		else 
		{
			$status = "OK";
			$return = priority_state($return,$ERRORS{OK});
		}
		$message = "$status : $curr_desc(Current device: $curr_serial) number session: ".$ses_val." $endline |'session'=".$ses_val.";".$warning.";".$critical."\n";

	}
	return $message;
}
sub get_vpn_state
{
	my $ipsectunsdown = 0;
	my $ipsectunsopen = 0;
	my $ActiveSSL = 0;
	my $ActiveSSLTunnel = 0;
    #si valeur par défaut on indique les valeurs VPN par défaut
    if($warning == 80)
    {
        $warning = 1;
        $critical= 2;
    }
	# Enumeration for the tunnel up/down states
	my %entitystate = ( 
		                  '1' => 'down',
		                  '2' => 'up' 
		                );
	my $status = "OK";
	my $result = $snmp->get_request($oid_ActiveSSL);
	if (!defined($result)) 
	{
		$message = "No SSL connection na\n";
	}
	else
	{
		$ActiveSSL = $$result{$oid_ActiveSSL};
		$result = $snmp->get_request($oid_ActiveSSLTunnel);
		$ActiveSSLTunnel = $$result{$oid_ActiveSSLTunnel};
	}
	#TUNNEL IPSEC
	$result = $snmp->get_table(Baseoid => $oid_VPNtunStatus );
	my $nbr_ipsec=0;
	foreach my $key ( keys %$result) 
	{	
		$nbr_ipsec++;
		my $vpn_state = $$result{$key};
		if($vpn_state == 2 || $vpn_state eq "up")
		{
			$ipsectunsopen++;
		}
		else
		{
			$ipsectunsdown++
		}
	}
    if ($ipsectunsdown >= $critical)
    {
        $return = priority_state($return,$ERRORS{CRITICAL}); 
        $status = "CRITICAL";
    }
	elsif ($ipsectunsdown >= $warning) 
    {
        $return = priority_state($return,$ERRORS{WARNING});
        $status = "WARNING";
    }
	$message = "$status : $curr_desc(Current device: $curr_serial) Active SSL-VPN Connections/Tunnels: " . $ActiveSSL."/".$ActiveSSLTunnel;  
 	$message = $message .": IPSEC Tunnels: Configured/Active: " . $nbr_ipsec . "/" . $ipsectunsopen. " |'ActiveSSL-VPN'=".$ActiveSSL." 'ActiveIPSEC'=".$ipsectunsopen;

	return $message;
}

