#!/usr/bin/perl -w
######################### check_snmp_hp_ilo #################
my $Version='1.0';
# Date : February 05 2018
# Author : Victor Marin
# Help : esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;
use List::MoreUtils qw(uniq);
my $PROGNAME='check_snmp_hp_ilo.pl';
my $TIMEOUT = 15;
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my ($opt_h,$opt_V);
my $domain = "UDP/IPV4"; # domain UDP/IPV6;UDP/IPV4;TCP/IPV6;TCP/IPV4
my $community = "public";
my $snmp_version = 2;
my ($hostname);
my $warning='70';
my $port='161';
my $critical='80';
my $result;
my $result_index;
#my @tab_value_read;
my ($snmp, $snmperror);
my $output = "";
my $target = "ALL";
my $val;
my $min_index = 0;
my $max_index = 100;
my $state = "OK";
my $return=$ERRORS{OK};

my $hp_descr 					= "1.3.6.1.2.1.1.1.0";

my $hp_cpus                     = "1.3.6.1.4.1.232.1.2.2.1.1.1";
my $hp_val_cpu                  = "1.3.6.1.4.1.232.1.2.2.1.1.6";

my $hp_cntrl                    = "1.3.6.1.4.1.232.3.2.2.1.1.1";
my $hp_val_cntrl 			    = "1.3.6.1.4.1.232.3.2.2.1.1.6";

my $hp_logical_drive 		    = "1.3.6.1.4.1.232.3.2.3.1.1.1";
my $hp_logical_drive_index 	    = "1.3.6.1.4.1.232.3.2.3.1.1.2";
my $hp_logical_drive_index_val  = "1.3.6.1.4.1.232.3.2.3.1.1.4";
my $hp_logical_drive_size 	    = "1.3.6.1.4.1.232.3.2.3.1.1.9";
my $hp_logical_drive_rebuild    = "1.3.6.1.4.1.232.3.2.3.1.1.12";

my $hp_physical_drive_cntrl     = "1.3.6.1.4.1.232.3.2.2.1.1.1";
my $hp_physical_drive           = "1.3.6.1.4.1.232.3.2.5.1.1.2";
my $hp_physical_drive_status    = "1.3.6.1.4.1.232.3.2.5.1.1.6";
my $hp_physical_drive_bay       = "1.3.6.1.4.1.232.3.2.5.1.1.5";
my $hp_physical_drive_model     = "1.3.6.1.4.1.232.3.2.5.1.1.3";
my $hp_physical_drive_Size      = "1.3.6.1.4.1.232.3.2.5.1.1.45";
my $hp_physical_drive_sn        = "1.3.6.1.4.1.232.3.2.5.1.1.51";

my $hp_smartcl          	    = "1.3.6.1.4.1.232.3.2.2.1.1.1";
my $hp_smartcl_index    	    = "1.3.6.1.4.1.232.3.2.5.1.1.2";
my $hp_smartcl_index_val	    = "1.3.6.1.4.1.232.3.2.5.1.1.57";

my $hp_psu_global_status        = "1.3.6.1.4.1.232.6.2.9.1";
my $hp_psu_condition            = "1.3.6.1.4.1.232.6.2.9.3.1.4";
my $hp_psu_bay					= "1.3.6.1.4.1.232.6.2.9.3.1.2";
my $hp_psu_status			    = "1.3.6.1.4.1.232.6.2.9.3.1.5";
my $hp_psu_redontant			= "1.3.6.1.4.1.232.6.2.9.3.1.9";

my $hp_temp_chassis				= "1.3.6.1.4.1.232.6.2.6.8.1.1";
my $hp_temp_index               = "1.3.6.1.4.1.232.6.2.6.8.1.2";
my $hp_temp_valCelsius       	= "1.3.6.1.4.1.232.6.2.6.8.1.4";
my $hp_temp_limit       		= "1.3.6.1.4.1.232.6.2.6.8.1.5";
my $hp_temp_sensor_type       	= "1.3.6.1.4.1.232.6.2.6.8.1.3";
my $hp_temp_status           = "1.3.6.1.4.1.232.6.2.6.8.1.6";
my $hp_temp_location            = "1.3.6.1.4.1.232.6.2.6.8.1.8";


my $hp_fan 						= "1.3.6.1.4.1.232.6.2.6.7.1.1";
my $hp_fan_index 				= "1.3.6.1.4.1.232.6.2.6.7.1.2";
my $hp_fan_index_val 			= "1.3.6.1.4.1.232.6.2.6.7.1.9";
my $hp_fan_locate				= "1.3.6.1.4.1.232.6.2.6.7.1.3";
my $hp_fan_speed				= "1.3.6.1.4.1.232.6.2.6.7.1.6";
my @arr_location_translate		= ("unknown","other","unknown","system","systemBoard","ioBoard","cpu","memory","storage","removableMedia","powerSupply","ambient","chassis","bridgeCard");
my @arr_sensors_location_translate	= ("other","unknown","system","systemBoard","ioBoard","cpu","memory","storage","removableMedia","powerSupply","ambient","chassis","bridgeCard");


sub print_help() {
	printf "check SNMP HP ilo via SNMP.\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)		Hostname to query - (required)\n";
	print " -D (--domain)		UDP/IPV6 UDP/IPV4 TCP/IPV6 TCP/IPV4 \n";
	print " -C (--community)	SNMP read community (defaults to public)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "		     2 for SNMP v2c (default)\n";
	print " -T	target value (default: ALL)\n";
	print " -w	warning value (default: 70)\n";
	print " -c	critical value (default: 80)\n";
	print " -V (--version)	Plugin version\n";
	print " -h (--help)	usage help\n";
	print "\n";
}

sub error() {
	print (" ERROR SNMP \n Verify your OID");
	#si l'host ne répond pas on blocke le script
	if (!defined($result))
	{
		if($snmp_version == 2)
		{
			($snmp, $snmperror) = Net::SNMP->session(
					-hostname  => $hostname,
					-domain    => $domain,
					-version   => 'snmpv1',
					-port      => $port,
					-community => $community
				);

			$result = $snmp->get_request("$hp_descr");
			if (!defined($result))
			{
				printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
				$snmp->close;
				exit $ERRORS{"UNKNOWN"};
			}
		}
		else
		{
			printf("UNKNOWN ERROR: Description table : %s.\n",
			$snmp->error);
			$snmp->close;
			exit $ERRORS{"UNKNOWN"};
		}
	}	
}

my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
}

sub print_usage() {
	 
	print "This snmpv2c plugin checks the hardware states on one HP server with the HP Insight System Management Agent\n";
	print "Usage : $0 <hostname or ip> <snmp community> <check> [device]\n";
	print "Usage : $0 <hostname or ip> <snmp community> <check> [device] [timeout en sec]\n";
	print "Usage : $0 <hostname or ip> <snmp community> <check> [min device] [max device] <timeout en sec>\n";
	print "The result also print <check name> <device> [details] : <state> => <description of this state>\n";
	print_commands();
	exit $ERRORS{"UNKNOWN"};
}

sub print_commands()
{
	print "Accepted commands :\n";
	print " cpu       : The Processor State\n";
	print " cntrl     : The overall condition of controllers, and any associated logical drives, physical drives, and array accelerators. FOR TESTING WITH MULTIPLES RAID CONTROLLEURS\n";
	print " raid      : Logical Drives Status. FOR TESTING WITH MULTIPLES RAID\n";
	print " disk    : Status of the physical drive\n";
	print " smart     : Physical Drive S.M.A.R.T Status.\n";
	print " pwr       : The overall condition of the fault tolerant power supply sub-system.\n";
	print " temp      : The Temperature sensor condition.\n";
	print " fan       : The condition of the fan.\n";
}

sub readOID {
    my($snmp,$oid) = @_;
	my $result = $snmp->get_request("$oid");
	return $$result{"$oid"};
}
########## FUNCTION SNMP ##########
sub check_cpus {
	my($snmp) = @_;
	my $result = $snmp->get_table(Baseoid => $hp_val_cpu);
	if(defined($result))
	{		
		foreach my $key ( keys %$result)
		{
			if($key =~ /([0-9]*)$/)
			{
				my $id =$1;
				my $val_cpu = $$result{$key};
				if($val_cpu == 1)
				{ 
					print "UNKNOWN: CPU $id The status of the CPU could not be determined."; 
					$return = priority_state($return,$ERRORS{UNKNOWN});
				}
				elsif($val_cpu == 2)
				{ 
					print "OK: CPU $id is OK \n"; 
				}
				elsif($val_cpu == 3)
				{ 
					print "WARNING: CPU $id Degraded => The CPU is in a pre-failure warrantee state."; 
					$return = priority_state($return,$ERRORS{WARNING});
				}
				elsif($val_cpu == 4)
				{ 
					print "CRITICAL: CPU $id Failed => The CPU is in a failed state."; 
					$return = priority_state($return,$ERRORS{CRITICAL});
				}
				elsif($val_cpu == 5)
				{ 
					print "CRITICAL: CPU $id Disabled => The CPU has been disabled during power-on-self-test.";
					$return = priority_state($return,$ERRORS{CRITICAL});
				}
				else   
				{ 
					print "UNKNOWN: CPU $id Unknown => The state of the CPU could not be determined."; 
					$return = priority_state($return,$ERRORS{UNKNOWN});
				}			
			}
		}
	}
	else
	{
		error();
		$return = priority_state($return,$ERRORS{UNKNOWN})
	}
}
sub check_cntrl {
	my($snmp) = @_;
	my $result = $snmp->get_table(Baseoid => $hp_val_cntrl);
	if(defined($result))
	{
		foreach my $key ( keys %$result)
		{
			if($key =~ /([0-9]*)$/)
			{
				my $id =$1;
				my $val_cntrl = $$result{$key};
				if($val_cntrl == 1)
				{ 
					print "UNKNOWN: cntrl $id Other => The status of the controller could not be determined."; 
					$return = priority_state($return,$ERRORS{UNKNOWN});
				}
				elsif($val_cntrl == 2)
				{ 
					print "OK: cntrl $id => This controller, and any associated logical drives, physical drives, and array accelerators functioning normally\n"; 
				}
				elsif($val_cntrl == 3)
				{ 
					print "WARNING: cntrl $id Degraded => One or more RAIDs in controller are in degraded state."; 
					$return = priority_state($return,$ERRORS{WARNING});
				}
				elsif($val_cntrl == 4)
				{ 
					print "CRITICAL: cntrl $id Failed => This controller, or any associated logical drives, physical drives, or array accelerators is in failed state."; 
					$return = priority_state($return,$ERRORS{CRITICAL});
				}
				else   
				{ 
					print "UNKNOWN: cntrl $id Unknown => The state of this controller could not be determined."; 
					$return = priority_state($return,$ERRORS{UNKNOWN});
				}
			}			
		}
	}
	else
	{
		error();
		$return = priority_state($return,$ERRORS{UNKNOWN})
	}
}
sub check_logicalDrive{	
	my($snmp) = @_;
	my $result = $snmp->get_table(Baseoid => "$hp_logical_drive_index_val");
	if(defined($result)) # && defined($result_index))
	{		
		foreach my $key ( keys %$result)
		{
			if($key =~ /([0-9]*)\.([0-9]*)$/)
			{
				my $cntrl =$1;
				my $driveid =$2;
				my $val_logicalDrive = $$result{$key};
				my $size = readOID($snmp,"$hp_logical_drive_size.$cntrl.$driveid");	
				my $name = "RAID on cntrl $cntrl number $driveid";
				if(defined $size && $size =~ /^\d+?$/)
				{
					$size = sprintf("%.2f",($size/1024));			
					$name = "$name $size GB";
				}
				if($val_logicalDrive == 1)
				{ 
					print "UNKNOWN: $name Other => The status of the logical drive could not be determined \n";
					$return = priority_state($return,$ERRORS{UNKNOWN});
				}
				elsif($val_logicalDrive == 2)
				{ 
					print "OK: $name => This logical drive is in normal operation mode\n"; 
				}		
				elsif($val_logicalDrive == 3)
				{ 
					print "CRITICAL: $name : Failed => More physical drives have failed than the fault tolerance mode of this logical drive can handle without data loss.\n";
					$return = priority_state($return,$ERRORS{CRITICAL});
				}
				elsif($val_logicalDrive == 4)
				{ 
					print "WARNING: $name : Unconfigured => This logical drive is not configured.\n";
					$return = priority_state($return,$ERRORS{WARNING});
				}
				elsif($val_logicalDrive == 5)
				{ 
					print "WARNING: $name : Recovering => The logical drive is using Interim Recovery Mode. In Interim Recovery Mode, at least one physical drive has failed, but this logical drive's fault tolerance mode lets the drive continue to operate with no data loss.\n";
					$return = priority_state($return,$ERRORS{WARNING});
				}
				elsif($val_logicalDrive == 6)
				{ 
					print "WARNING: $name : Ready Rebuild => This logical drive is ready for Automatic Data Recovery. The physical drive that failed has been replaced, but this logical drive is still operating in Interim Recovery Mode.\n";
					$return = priority_state($return,$ERRORS{WARNING});
				}
				elsif($val_logicalDrive == 7)
				{ 
					my $output = readOID($snmp,"$hp_logical_drive_rebuild.$cntrl.$driveid");	
					print "WARNING: $name : Rebuilding $output% => This logical drive is currently doing Automatic Data Recovery. During Automatic Data Recovery, fault tolerance algorithms restore data to the replacement drive. \n";
					$return = priority_state($return,$ERRORS{WARNING});
				}
				elsif($val_logicalDrive == 8)
				{ 
					print "WARNING: $name : Wrong Drive => The wrong physical drive was replaced after a physical drive failure.\n";
					$return = priority_state($return,$ERRORS{WARNING});
				}
				elsif($val_logicalDrive == 9)
				{ 
					print "WARNING: $name : Bad Connect => A physical drive is not responding.\n";
					$return = priority_state($return,$ERRORS{WARNING});
				}
				elsif($val_logicalDrive == 10)
				{ 
					print "CRITICAL: $name Overheating => The drive array enclosure that contains the logical drive is overheating. The drive array is still functioning, but should be shutdown. \n";
					$return = priority_state($return,$ERRORS{CRITICAL});
				}
				elsif($val_logicalDrive == 11)
				{ 
					print "CRITICAL: $name Shutdown => The drive array enclosure that contains the logical drive has overheated. The logical drive is no longer functioning. \n";
					$return = priority_state($return,$ERRORS{CRITICAL});
				}
				elsif($val_logicalDrive == 12)
				{ 
					print "OK: $name : Expanding => This logical drive is currently doing Automatic Data Expansion. During Automatic Data Expansion, fault tolerance algorithms redistribute logical drive data to the newly added physical drive. \n";
				}
				elsif($val_logicalDrive == 13)
				{ 
					print "CRITICAL: $name Not Available => This logical drive is currently unavailable. If a logical drive is expanding and the new configuration frees additional disk space, this free space can be configured into another logical volume.  If this is done, the new volume will be set to not available. \n";
					$return = priority_state($return,$ERRORS{CRITICAL});
				}
				elsif($val_logicalDrive == 14)
				{ 
					print "WARNING: $name : Not Available => This logical drive is ready for Automatic Data Expansion. The logical drive is in the queue for expansion.\n";
					$return = priority_state($return,$ERRORS{WARNING});
				}
				else
				{
					print "UNKNOWN: $name Unknown => The state of this logical drive could not be determined. \n"; 
					$return = priority_state($return,$ERRORS{UNKNOWN});
				}
			}
		}
	}
	else
	{
		print ("UNKNOWN: No logical drive found \n");
		$return = priority_state($return,$ERRORS{UNKNOWN})
	}	
}
sub check_physicalDrive{
	my ($snmp) = @_;
	my $result = $snmp->get_table(Baseoid => $hp_physical_drive_bay);	
	if(defined($result))
	{			
		foreach my $key ( keys %$result)
		{
			if($key =~ /([0-9]*)\.([0-9]*)$/)
			{
				my $cntrl =$1;
				my $driveid =$2;
	
				my $bay = $$result{$key};
				my $model = readOID($snmp,"$hp_physical_drive_model.$cntrl.$driveid");
				my $size = readOID($snmp,"$hp_physical_drive_Size.$cntrl.$driveid");
				my $sn = readOID($snmp,"$hp_physical_drive_sn.$cntrl.$driveid");
				my $name = "$model ($size MB)";
				my $val_physicalDrive = readOID($snmp,"$hp_physical_drive_status.$cntrl.$driveid");
				if($val_physicalDrive == 1)
				{ 
					print "UNKNOWN: disque $cntrl.$driveid name $name SN:$sn Baie:$bay => Other: The instrument agent does not recognize the drive.  You may need to upgrade your instrument agent and/or driver software. \n";
					$return = priority_state($return,$ERRORS{UNKNOWN});
				}
				elsif($val_physicalDrive == 2)
				{ 
					print "OK: disque $cntrl.$driveid name $name SN: $sn Baie:$bay => OK \n";
				}	
				elsif($val_physicalDrive == 3)
				{ 
					print "CRITICAL: disque $cntrl.$driveid name $name SN: $sn Baie:$bay => Failed: This drive is no longer operating and should be replaced. \n"; 
					$return = priority_state($return,$ERRORS{CRITICAL});
				}	
				elsif($val_physicalDrive == 4)
				{ 
					print "WARNING: $cntrl.$driveid name $name SN: $sn Baie:$bay => PredictiveFailure: This drive has a predictive failure error and should be replaced.\n";
					$return = priority_state($return,$ERRORS{WARNING});
				}	
				else
				{
					print "UNKNOWN: disque $cntrl.$driveid <$name>$sn Baie:$bay => The state of this drive could not be determined. \n"; 
					$return = priority_state($return,$ERRORS{UNKNOWN});
				}	
			}			
		}
	}	
	else
	{
		print ("UNKNOWN: No physical drive found \n");
		$return = priority_state($return,$ERRORS{UNKNOWN});
	}
}
sub check_smartcl {	
	my ($snmp) = @_;
			
	my $result = $snmp->get_table(Baseoid => $hp_smartcl_index_val);
	if(defined($result))
	{		
		foreach my $key ( keys %$result)
		{
			if($key =~ /([0-9]*)\.([0-9]*)$/)
			{
				my $cntrl =$1;
				my $driveid =$2;
				my $bay = readOID($snmp,"$hp_physical_drive_bay.$cntrl.$driveid");
				my $model = readOID($snmp,"$hp_physical_drive_model.$cntrl.$driveid");
				my $sn = readOID($snmp,"$hp_physical_drive_sn.$cntrl.$driveid");
				my $name = "$model ";
				my $val_smartcl = $$result{$key};
				if($val_smartcl == 1)
				{ 
					print "UNKNOWN: smartcl $cntrl.$driveid name $name SN:$sn Baie:$bay => Other: The agent is unable to determine he status of S.M.A.R.T predictive failure monitoring for this drive. \n";
					$return = priority_state($return,$ERRORS{UNKNOWN});
				}
				elsif($val_smartcl == 2)
				{ 
					 print "OK: smartcl $cntrl.$driveid name $name SN:$sn Baie:$bay=> OK \n";
				}	
				elsif($val_smartcl == 3)
				{ 
					print "WARNING: smartcl $cntrl.$driveid name $name SN:$sn Baie:$bay => replaceDrive: This drive has a S.M.A.R.T predictive failure error and should be replaced. \n"; 
					$return = priority_state($return,$ERRORS{CRITICAL});
				}		
				else
				{
					print "UNKNOWN: smartcl $$cntrl.$driveid name $name SN:$sn Baie:$bay => Unknown: The state of this drive could not be determined. \n"; 
					$return = priority_state($return,$ERRORS{UNKNOWN});
				}
			}		
		}
	}
	else
	{
		print ("UNKNOWN: No physical drive S.M.A.R.T found \n");
		$return = priority_state($return,$ERRORS{UNKNOWN})
	}	
}
sub check_alim {
	my ($snmp) = @_;
	
	my $state = "OK";
	my $global_psu_status = readOID($snmp,$hp_psu_global_status);
	my $resultglobalstatus = $snmp->get_table(Baseoid => $hp_psu_global_status);
	foreach my $keypsy ( keys %$resultglobalstatus)
	{
		my $global_psu_status = $$resultglobalstatus{$keypsy};
		if ($global_psu_status==1)
		{
			print "UNKNOWN: global power Supply is unknown state\n";
			$return = priority_state($return,$ERRORS{UNKNOWN})
		}
	}
	my $result = $snmp->get_table(Baseoid => $hp_psu_condition);
	if(defined($result))
	{		
		foreach my $key ( keys %$result)
		{	
			if($key =~ /([0-9]*)\.([0-9]*)$/)
			{
				my $chassisid =$1;
				my $psuid =$2;
				my $bay = readOID($snmp,"$hp_psu_bay.$chassisid.$psuid");
				my $redondante = readOID($snmp,"$hp_psu_redontant.$chassisid.$psuid");
				my $msg = " ";
				if($redondante==2)
				{
					$msg = "Non-redundant ";
				}
				elsif($redondante==3)
				{
					$msg = "Redundant ";
				}
				my $psu_statut = $$result{$key};
				my $psu_message = readOID($snmp,"$hp_psu_status.$chassisid.$psuid");
				if($psu_message == 1)
				{ 
					$psu_message = "No Error";
				}
				elsif($psu_message == 2)
				{ 
					 $psu_message = "generalFailure";
				}	
				elsif($psu_message == 3)
				{ 
					$psu_message = "bistFailure";
				}	
				elsif($psu_message == 4)
				{ 
					$psu_message = "fanFailure";
				}	
				elsif($psu_message == 5)
				{ 
					$psu_message = "tempFailure";
				}	
				elsif($psu_message == 6)
				{ 
					$psu_message = "interlockOpen";
				}	
				elsif($psu_message == 7)
				{ 
					$psu_message = "epromFailed";
				}	
				elsif($psu_message == 8)
				{ 
					$psu_message = "vrefFailed";
				}	
				elsif($psu_message == 9)
				{ 
					$psu_message = "dacFailed";
				}
				elsif($psu_message == 10)
				{ 
					$psu_message = "ramTestFailed";
				}
				elsif($psu_message == 11)
				{ 
					$psu_message = "voltageChannelFailed";
				}	
				elsif($psu_message == 12)
				{ 
					$psu_message = "orringdiodeFailed";
				}	
				elsif($psu_message == 13)
				{ 
					$psu_message = "brownOut";
				}	
				elsif($psu_message == 14)
				{ 
					$psu_message = "giveupOnStartup";
				}	
				elsif($psu_message == 15)
				{ 
					$psu_message = "nvramInvalid";
				}	
				elsif($psu_message == 16)
				{ 
					$psu_message = "calibrationTableInvalid";
				}		
				my $psu_state_msg="OK";
				if($psu_statut==3)
				{
					$psu_state_msg= "WARNING";
					$return = priority_state($return,$ERRORS{WARNING});
					print "$psu_state_msg: Power Supply $redondante on $bay is in error $psu_message \n";
				}
				elsif($psu_statut==3)
				{
					$psu_state_msg= "CRITICAL";
					$return = priority_state($return,$ERRORS{CRITICAL});
					print "$psu_state_msg: Power Supply $redondante on $bay is in critical error $psu_message \n";
				}
				else
				{
					print "$psu_state_msg: Power Supply $msg on bay $bay ==> No error. \n";
				}
			}
		}
	}
}
sub check_temp {
	 my ($snmp) = @_;
		
	my $maxtemp_cpu = 0;
	my $maxtemp_mem = 0;
	my $maxtemp_ambient = 0;
	my $maxtemp_sys = 0;    
    my $result = $snmp->get_table(Baseoid =>"$hp_temp_valCelsius");
	if(defined($result))
	{		     
		foreach my $key( keys %$result)
		{
			if($key =~ /([0-9]*)\.([0-9]*)$/)
			{
				my $temp = $$result{$key};
				my $chassisid =$1;
				my $id =$2;
				my $limit = readOID($snmp,"$hp_temp_limit.$chassisid.$id");
				my $realstatus = readOID($snmp,"$hp_temp_status.$chassisid.$id");
				my $type = readOID($snmp,"$hp_temp_sensor_type.$chassisid.$id");
				my $sensors_state_msg= "OK";
				my $sensors_msg= "";
				if($realstatus == 1)
				{
					$return = priority_state($return,$ERRORS{UNKNOWN});
					$sensors_state_msg= "UNKNOWN";
					$sensors_msg = "Temperature status could not be determined.";
				}
				elsif($realstatus == 2)
				{
					$return = priority_state($return,$ERRORS{OK});
				}
				elsif($realstatus == 3)
				{
					$return = priority_state($return,$ERRORS{WARNING});
					$sensors_state_msg= "WARNING";
					$sensors_msg = "Degraded => The temperature sensor is outside of normal operating range.";
				}
				elsif($realstatus == 4)
				{
					$return = priority_state($return,$ERRORS{CRITICAL});
					$sensors_state_msg= "CRITICAL";
					$sensors_msg = "Failed  => The temperature sensor detects a condition that could permanently damage the system.";
				}
				else
				{
					$return = priority_state($return,$ERRORS{UNKNOWN});
					$sensors_state_msg= "UNKNOWN";
					$sensors_msg = "Temperature status could not be determined.";
				}
				print "$sensors_state_msg: sensor $id (".$arr_sensors_location_translate[$type].") Temperature = $temp Limit = $limit. $sensors_msg\n";
				#PARTIE POUR LE GRAPHIQUE
				if($type ==3) #système
				{
					if($temp>$maxtemp_sys)
					{
						$maxtemp_sys=$temp;
					}
				}
				elsif($type ==6) #CPU
				{
					if($temp>$maxtemp_cpu)
					{
						$maxtemp_cpu=$temp;
					}
				}
				elsif($type ==7)#MEMOIRE
				{
					if($temp>$maxtemp_mem)
					{
						$maxtemp_mem=$temp;
					}
				}
				elsif($type ==11)#AMBIENT
				{
					if($temp>$maxtemp_ambient)
					{
						$maxtemp_ambient=$temp;
					}
				}
				
			}
		}
		print "|cpu=$maxtemp_cpu,memory=$maxtemp_mem,ambient=$maxtemp_ambient,system=$maxtemp_sys";
	}
	else
	{
		print ("UNKNOWN: No sensors found \n");
		$return = priority_state($return,$ERRORS{UNKNOWN})
	}	
}
sub check_fan {
	my ($snmp) = @_;

    my $result = $snmp->get_table(Baseoid =>"$hp_fan_index_val");	 
	if(defined($result))
	{		
		foreach my $key ( keys %$result)
		{
				if($key =~ /([0-9]*)\.([0-9]*)$/)
				{
				my $chassisid =$1;
				my $id =$2;
				my $val_fan = $$result{$key};
				my $index_location = readOID($snmp,"$hp_fan_locate.$chassisid.$id");
				my $location = $arr_location_translate[$index_location];
				my $speed = readOID($snmp,"$hp_fan_speed.$chassisid.$id");
				if($speed==1)
				{
					$speed="other"; 
				}
				elsif($speed == 2)
				{ 
					$speed = "normal";
				}	
				elsif($speed == 3)
				{ 
					$speed = "high";
				}	
				else
				{
					$speed="unknown";
				}
				if($val_fan==1)
				{
					print "OK: fan $id $location=$speed : OK\n";
				}
				elsif($val_fan == 2)
				{ 
					print "OK: fan $id $location=$speed : OK\n";
				}	
				elsif($val_fan == 3)
				{ 
					print "WARNING: fan $id $location=$speed : degraded => A redundant fan is not operating.\n"; 
					$return = priority_state($return,$ERRORS{WARNING});
				}	
				elsif($val_fan == 4)
				{ 
					print "CRITICAL: fan $id $location=$speed : failed => A non-redundant fan is not operating.\n"; 
					$return = priority_state($return,$ERRORS{CRITICAL});
				}	
				else
				{
					print "UNKNOWN: fan $id => Temperature could not be determined.\n"; 
					$return = priority_state($return,$ERRORS{UNKNOWN});
				}
			}
		}
	}
}
########## MAIN ##########

&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,        "version"        => \$opt_V,
	"D=s" => \$domain,       "domain=s"       => \$domain,
	"h"   => \$opt_h,        "help"           => \$opt_h,
	"C=s" => \$community,    "community=s"    => \$community,
	"H=s" => \$hostname,     "hostname=s"     => \$hostname,
	"v=i" => \$snmp_version, "snmp_version=i" => \$snmp_version,
	"w=i" => \$warning,      "warning=i"      => \$warning,
	"c=i" => \$critical,     "critical=i"     => \$critical,
	"T=s" => \$target,		 "target=s"		  => \$target,
	"t=s" => \$TIMEOUT,		 "timeout=s"	  => \$TIMEOUT,
);

# -h & --help print help
if($opt_h) { print_help(); exit $ERRORS{'OK'}; }
if(!$hostname) {print_help(); exit $ERRORS{'OK'}; }
if(!$community) {print_help(); exit $ERRORS{'OK'}; }
if(!$domain) {print_help(); exit $ERRORS{'OK'}; }

if ($snmp_version == 2)
{
	($snmp, $snmperror) = Net::SNMP->session(
			-hostname   => $hostname,
			-version    => 'snmpv2c',
			-domain     => $domain,
			-port       => $port,
			-community  => $community,
			-timeout    => 2
		);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
			-hostname  => $hostname,
			-version   => 'snmpv1',
			-domain    => $domain,
			-port      => $port,
			-community => $community,
			-timeout   => 2
		);
}

if(!defined($snmp))
{
	print " Error SNMP not defined !!! \n";
	exit $ERRORS{"UNKNOWN"};
}
else
{		
	$result = $snmp->get_request("$hp_descr");
	if (!defined($result))
	{
		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
		$snmp->close;
		exit $ERRORS{"UNKNOWN"};
	}
	my $model = $$result{$hp_descr};
	print "$model \n";	
	
	## REGEX POUR SAVOIR QUELLE VERSION ILO
	my $ilo = 0;
	my $version = 0;
	my $limit_version = "2.46";
    $model=~ /(integrated.*Lights.*out|ilo)\D*(\d)\s(\S+)/i;
    if (defined $2)
    {
    	$ilo = $2;
        $version = $3;
    }
	$target = uc $target;
	if($target eq "CPU")
	{
		check_cpus($snmp);		
	}
	elsif($target eq "CNTRL")
	{
		check_cntrl($snmp);	
	}
	elsif($target eq "RAID")
	{
		check_logicalDrive($snmp);	
	}
	elsif($target eq "DISK-ONLY")
	{
		check_physicalDrive($snmp);	
	}
	elsif($target eq "DISK")
	{
		check_physicalDrive($snmp);	
		check_smartcl($snmp);	
	}
	elsif($target eq "SMART")
	{
		check_smartcl($snmp);	
	}
	elsif($target eq "DISKHEALTH")
	{
		check_cntrl($snmp);	
		check_logicalDrive($snmp);	
		check_physicalDrive($snmp);	
		check_smartcl($snmp);	
	}
    elsif($target eq "POWER")
    { 
    	check_alim($snmp);
    }
    elsif($target eq "TEMP")
    { 
    	check_temp($snmp);
    }
    elsif($target eq "FAN")
    { 
    	check_fan($snmp);
    }
    elsif($target eq "SYSTEMHEALTH")
    { 
		check_cpus($snmp);
    	check_alim($snmp);
    	check_fan($snmp);
    }
}
exit $return;

