#!/usr/bin/perl -w 
############################## check_snmp_hp5400_temp.pl ########
my $Version='1.0';
# Date : 19 février 2019
# Author  : Nicolas Biersart
# Help : support@esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;

my $PROGNAME='check_snmp_hp_switch.pl';
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my $return=0;
my ($opt_h,$opt_V);
my $community = "public";
my $hostname;
my $snmp_version = 2;
my $warning='';
my $critical='';
my $type = 'env';
my $oid_sysObjectOID = "1.3.6.1.2.1.1.2.0";

#procurve switchs
my $oid_procurve_operating_descr    	= "1.3.6.1.4.1.11.2.14.11.1.2.6.1.7"; 	# Component description
my $oid_procurve_operating_state    	= "1.3.6.1.4.1.11.2.14.11.1.2.6.1.4"; 	# Operating status of component - Unknown(1), Bad(2), Warning(3), Good(4), NotPresent(5)
my %procurve_operating_status_text  = ( "0"  => "--Invalid--","1" => "Unknown","2"  => "Bad","3"  => "Warning","4"  => "Good","5"  => "NotPresent");
my %procurve_operating_status = ( "0"  => "UNKNOWN","1"  => "UNKNOWN","2" => "WARNING","3"  => "CRITICAL","4"  => "OK","5"  => "OK");
#CPU
my $oid_procurve_cpu = "1.3.6.1.4.1.11.2.14.11.5.1.9.6.1.0"; # Procurve CPU Counter
# MEM
my $oid_procurve_mem_index	    = "1.3.6.1.4.1.11.2.14.11.5.1.1.2.2.1.1.1"; # memory slot index
my $oid_procurve_mem_total	    = "1.3.6.1.4.1.11.2.14.11.5.1.1.2.2.1.1.5"; # Total Bytes
my $oid_procurve_mem_free		= "1.3.6.1.4.1.11.2.14.11.5.1.1.2.2.1.1.6"; # Free Bytes
my $oid_procurve_mem_free_seg	= "1.3.6.1.4.1.11.2.14.11.5.1.1.2.2.1.1.3"; # Free segments
#ARUBA
my $oid_hpSystemAirName = "1.3.6.1.4.1.11.2.14.11.1.2.8.1.1.2";
my $oid_hpCurrentTemp = "1.3.6.1.4.1.11.2.14.11.1.2.8.1.1.3";
my $oid_hpThresholdTemp = "1.3.6.1.4.1.11.2.14.11.1.2.8.1.1.7";
#HPE 5130
my $oid_sensorType = "1.3.6.1.2.1.99.1.1.1.1";	#8 Celcius
my $oid_sensorValue = "1.3.6.1.2.1.99.1.1.1.4";	#Value
my $oid_sensorUnit = "1.3.6.1.2.1.99.1.1.1.2"; 	#9 Unit
my $oid_hpe_sensorName = "1.3.6.1.2.1.47.1.1.1.1.7"; 
#Sensors
my $oid_hpe_sensorStatus="1.3.6.1.4.1.25506.2.6.1.1.1.1.3";
my %hpe_sensorStatus_text  = ( "0"  => "--Invalid--","1" => "notSupported","2"  => "disabled","3"  => "enabled","4"  => "dangerous");
my %hpe_sensorStatus_status = ( "0"  => "UNKNOWN","1"  => "UNKNOWN","2" => "OK","3"  => "OK","4"  => "CRITICAL");
my $oid_hpe_sensorTemperature="1.3.6.1.4.1.25506.2.6.1.1.1.1.12";
my $oid_hpe_sensorTemperatureThreshold="1.3.6.1.4.1.25506.2.6.1.1.1.1.13";
my $oid_hpe_sensorTemperatureLowerThreshold="1.3.6.1.4.1.25506.2.6.1.1.1.1.21";
#CPU&mem
my $oid_hpe_cpu_usage = "1.3.6.1.4.1.25506.2.6.1.1.1.1.6";
my $oid_hpe_mem_usage = "1.3.6.1.4.1.25506.2.6.1.1.1.1.8";

# SNMPv3 specific
my $o_login=	undef;		# Login for snmpv3
my $o_passwd=	undef;		# Pass for snmpv3
my $v3protocols=undef;	# V3 protocol list.
my $o_authproto='md5';		# Auth protocol
my $o_privproto='des';		# Priv protocol
my $o_privpass= undef;		# priv password

sub getsnmpOID {
    my($snmp,$oid) = @_;
	my $result = $snmp->get_request("$oid");
	return $$result{"$oid"};
}

sub print_help() {
	printf "check snmp HP switch.\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)     Hostname to query - (required)\n";
	print " -C (--community)    SNMP read community (defaults to public)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "                     2 for SNMP v2c (default)\n";
	print " -T                  Test type (cpu,mem,env)\n";
	print " -w                  warning threshold\n";
	print " -c                  critical threshold\n";
	print " -V (--version)      Plugin version\n";
	print " -h (--help)         usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>] [-w warning] [-c critical]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}
my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
} 

#
#MAIN
#
&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"C=s" => \$community,	"community=s" => \$community,
	"H=s" => \$hostname,	"hostname=s" => \$hostname,
	"v=i" => \$snmp_version,"snmp_version=i" => \$snmp_version,
    "T=s" => \$type,"type=s" => \$type,
	"w=i" => \$warning,"warning=i" => \$warning,
	"c=i" => \$critical,"critical=i" => \$critical,
	'l=s'	=> \$o_login,		'login=s'	=> \$o_login,
	'x=s'	=> \$o_passwd,		'passwd=s'	=> \$o_passwd,
	'X=s'	=> \$o_privpass,		'privpass=s'	=> \$o_privpass,
	'L=s'	=> \$v3protocols,		'protocols=s'	=> \$v3protocols,   
);
# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }
if (!$hostname) { print_help(); exit $ERRORS{'UNKNOWN'}; }
my ($snmp, $snmperror);
if (defined($o_login) || defined($o_passwd)) 
{
    $snmp_version =3;
}
if ($snmp_version == 3) 
{
    my @v3proto=split(/,/,$v3protocols);
    if ((defined ($v3proto[0])) && ($v3proto[0] ne "")) {$o_authproto=$v3proto[0];	}	# Auth protocol
    if (defined ($v3proto[1])) {$o_privproto=$v3proto[1];	}	# Priv  protocol
    if (!defined ($o_privpass)) 
    {
        ($snmp, $snmperror) = Net::SNMP->session(
          -hostname   	=> $hostname,
          -version		=> '3',
          -username		=> $o_login,
          -authpassword	=> $o_passwd,
          -authprotocol	=> $o_authproto,
          -timeout          => 2
        );  
    }
    else 
    {
        ($snmp, $snmperror) = Net::SNMP->session(
          -hostname   	=> $hostname,
          -version		=> '3',
          -username		=> $o_login,
          -authpassword	=> $o_passwd,
          -authprotocol	=> $o_authproto,
          -privpassword	=> $o_privpass,
	      -privprotocol => $o_privproto,
          -timeout          => 5
        );
    }
}
elsif ($snmp_version == 2) 
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv2c',
		-community => $community,
		-timeout=>2
	);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv1',
		-community => $community,
		-timeout=>2
	);
}
#on interroge en SNMP pour récupérer les informations 
my $result = $snmp->get_request("$oid_sysObjectOID");


#si l'host ne répond pas on blocke le script
if (!defined($result)) {
	if($snmp_version == 2)
	{
		($snmp, $snmperror) = Net::SNMP->session(
			-hostname => $hostname,
			-version => 'snmpv1',
			-community => $community
		);
		$result = $snmp->get_request("$oid_sysObjectOID");
		if (!defined($result)) 
		{
	   		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
	   		$snmp->close;
	  		 exit $ERRORS{"UNKNOWN"};
		}
	}
	else
	{
		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
		$snmp->close;
		 exit $ERRORS{"UNKNOWN"};
	}
}
my $Sysoid_value = $$result{"$oid_sysObjectOID"};
if($Sysoid_value =~/3\.6\.1\.4\.1\.([0-9]*)/)
{
    $Sysoid_value = $1;
}
# test des paramètre par défaut
if($warning eq '' )
{
    if($type eq 'cpu' || $type eq 'mem')
    {
        $warning=80;
    }
}
if($critical eq '')
{
    if($type eq 'cpu'|| $type eq 'mem')
    {
        $critical=80;
    }
}
#
#HP procurve switch
#
if($Sysoid_value == 11)
{
    if($type eq 'env')
    {
        $result = $snmp->get_table(Baseoid => $oid_procurve_operating_descr);
        foreach my $key ( keys %$result) 
        {	
	        #on récupère l'index
	        if($key=~ /([0-9]*)$/ )
	        {
                my $index =$1;
		        my $descr =$$result{"$key"};
                my $status = getsnmpOID($snmp,"$oid_procurve_operating_state.$index");
                my $state = $procurve_operating_status{$status};
                print "$state : $descr is ".$procurve_operating_status_text{$status}."\n";
                $return = priority_state($return,$ERRORS{$state});
            }
        } 
        $result = $snmp->get_table(Baseoid => $oid_hpSystemAirName);
        foreach my $key ( keys %$result) 
        {	
	        #on récupère l'index
	        if($key=~ /([0-9]*)$/ )
	        {
                my $index =$1;
		        my $descr =$$result{"$key"};
                my $temps = getsnmpOID($snmp,"$oid_hpCurrentTemp.$index");
                $temps =~s/C//g;
                my $thresh = getsnmpOID($snmp,"$oid_hpThresholdTemp.$index");
                $thresh =~s/C//g;
                my $state='OK';
                my $msg = "< $thresh C";
                if( $temps >= $thresh)
                {
                     $state='CRITICAL';
                    $msg = "> $thresh C";
                }
                print "$state : $descr temperature is $temps $msg\n";
                $return = priority_state($return,$ERRORS{$state});
            }
        }       



    }
    elsif($type eq 'cpu')
    {
        my $state = 'OK';
        my $threshold = getsnmpOID($snmp,"$oid_procurve_cpu"); 
        my $message="(< $warning)";
        if($threshold >= $critical)
        {
            $state='CRITICAL';
            $return = priority_state($return,$ERRORS{CRITICAL});
            $message="(> $critical)";
        }
        elsif($threshold >= $warning)
        {
            $state='WARNING';
            $return = priority_state($return,$ERRORS{WARNING});
            $message="(> $warning)";
        }
        printf("%s : CPU used %.1f%% %s",$state,$threshold,$message);
    }
    elsif($type eq 'mem')
    {
        $result = $snmp->get_table(Baseoid => $oid_procurve_mem_total);
        foreach my $key ( keys %$result) 
        {	
	        #on récupère l'index
	        if($key=~ /([0-9]*)$/ )
	        {
                my $index =$1;
		        my $total =$$result{"$key"};
		        my $memory = getsnmpOID($snmp,"$oid_procurve_mem_free.$index"); 
                my $state = 'OK';
                my $threshold = 100 - ($memory / $total)*100;
                my $message="(< $warning)";
                if($threshold >= $critical)
                {
                    $state='CRITICAL';
                    $return = priority_state($return,$ERRORS{CRITICAL});
                    $message="(> $critical)";
                }
                elsif($threshold >= $warning)
                {
                    $state='WARNING';
                    $return = priority_state($return,$ERRORS{WARNING});
                    $message="(> $warning)";
                }              
                printf("%s :Slot %s:%.0f%% %s",$state,$index,$threshold,$message);
               # print "$state : Slot $index: is $threshold\n";
               # $return = priority_state($return,$ERRORS{$state});
            }
        }  
       
    }
}
#
#HPE 5130 & co
#
elsif($Sysoid_value == 25506)
{
    if($type eq 'env')
    {
        my $nbr_sensors=0;
        $result = $snmp->get_table(Baseoid => $oid_hpe_sensorName);
        foreach my $key ( keys %$result) 
        {	
            my $state = 'OK';
	        #on récupère l'index
	        if($key=~ /([0-9]*)$/ )
	        {
                my $index =$1;
		        my $descr =$$result{"$key"};
                if($descr =~ /FAN/  ||$descr =~ /PSU/)
                {
                    
		            my $status = getsnmpOID($snmp,"$oid_hpe_sensorStatus.$index"); 
                    $state = $hpe_sensorStatus_status{$status};
                    print "$state : $descr is ".$hpe_sensorStatus_text{$status}."\n";
                    $return = priority_state($return,$ERRORS{$state});
                }
                elsif($descr =~ /SENSOR/)
                {
                    $nbr_sensors++;
                    my $temp = getsnmpOID($snmp,"$oid_hpe_sensorTemperature.$index");  
                    my $tempthreshold = getsnmpOID($snmp,"$oid_hpe_sensorTemperatureThreshold.$index");  
                    my $templower =  getsnmpOID($snmp,"$oid_hpe_sensorTemperatureLowerThreshold.$index");  
                    my $message=""; 
                    if($temp >= $tempthreshold)
                    {
                        $state="CRITICAL";
                        $message="high temperature $temp > $tempthreshold";
                    }
                    elsif($temp <= $templower)
                    {
                        $state="CRITICAL";
                        $message="lower temperature $temp < $templower";
                    }
                    else
                    {
                        $state="OK";
                        $message="normal $temp < $tempthreshold";
                    }
                    print "$state : $descr is $message\n";
                    $return = priority_state($return,$ERRORS{$state});
                }
            }
        } 
        if($nbr_sensors == 0)
        {
            print "OK : No sensors found\n";
            $return = priority_state($return,$ERRORS{OK});
        }    
    }
    elsif($type eq 'cpu')
    {
        $result = $snmp->get_table(Baseoid => $oid_hpe_sensorName);
        my $cpuusage =-1;
        foreach my $key ( keys %$result) 
        {	
	        #on récupère l'index
	        if($key=~ /([0-9]*)$/ )
	        {
                my $index =$1;
		        my $descr =$$result{"$key"};
                if($descr =~ /Board/ || $descr =~ /Level 1 Virtual Module/)
                {
                    $cpuusage = getsnmpOID($snmp,"$oid_hpe_cpu_usage.$index");  
                    last;
                }
            }
        }
        if($cpuusage != -1 )
        {
            my $state = "OK";
            my $message="(< $warning)";
            if($cpuusage >= $critical)
            {
                $state='CRITICAL';
                $return = priority_state($return,$ERRORS{CRITICAL});
                $message="(> $critical)";
            }
            elsif($cpuusage >= $warning)
            {
                $state='WARNING';
                $return = priority_state($return,$ERRORS{WARNING});
                $message="(> $warning)";
            }
            printf("%s : CPU used %.1f%% %s",$state,$cpuusage,$message);
        }
        else
        {
            print "UNKNWON: CPU not found";
            $snmp->close;
            exit $ERRORS{"UNKNOWN"};
        }
    }
    elsif($type eq 'mem')
    {
        $result = $snmp->get_table(Baseoid => $oid_hpe_sensorName);
        my $memusage =-1;
        foreach my $key ( keys %$result) 
        {	
	        #on récupère l'index
	        if($key=~ /([0-9]*)$/ )
	        {
                my $index =$1;
		        my $descr =$$result{"$key"};
                if($descr =~ /Board/ || $descr =~ /Level 1 Virtual Module/)
                {
                    $memusage = getsnmpOID($snmp,"$oid_hpe_mem_usage.$index");  
                    last;
                }
            }
        }
        if($memusage != -1 )
        {
            my $state = "OK";
            my $message="(< $warning)";
            if($memusage >= $critical)
            {
                $state='CRITICAL';
                $return = priority_state($return,$ERRORS{CRITICAL});
                $message="(> $critical)";
            }
            elsif($memusage >= $warning)
            {
                $state='WARNING';
                $return = priority_state($return,$ERRORS{WARNING});
                $message="(> $warning)";
            }
            printf("%s :Slot %s:%.0f%% %s",$state,1,$memusage,$message);
        }
        else
        {
            print "UNKNWON: CPU not found";
            $snmp->close;
            exit $ERRORS{"UNKNOWN"};
        }
    }
}
else
{
    print "OID is unknown. Model not support contact your support\@esia-sa.com\n";
    $snmp->close;
    exit $ERRORS{"UNKNOWN"};
}
exit($return);
