#!/usr/bin/perl -w

# basic requirements
use strict;
use Getopt::Long;
use Net::SNMP;

my $progname= "check_snmp_hwg.pl";
my $snmp_version  = "1";
# options
my $opt_community = "public";
my $opt_host= "127.0.0.1";
my $opt_sensor;
my $opt_contact;
my $opt_warning=25;
my $opt_critical=30;
my $opt_calibre=0;
my $opt_help;
my $opt_man;
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
# oids
my $enterprise = '.1.3.6.1.4.1.21796';
my $oid_poseidon_1wireid = '.1.3.6.1.4.1.21796.3.3.3.1.8';
my $oid_poseidon_sensName = '.1.3.6.1.4.1.21796.3.3.3.1.2';# POSEIDON-MIB::sensName
my $oid_poseidon_sensValue = '.1.3.6.1.4.1.21796.3.3.3.1.6';# POSEIDON-MIB::sensName
my $oid_poseidon_unit = '.1.3.6.1.4.1.21796.3.3.3.1.9';# POSEIDON-MIB::sensName

my $oid_ste2_1wireid  = '.1.3.6.1.4.1.21796.4.9.3.1.8';
my $oid_ste2_sensName = '.1.3.6.1.4.1.21796.4.9.3.1.2';
my $oid_ste2_sensValue= '.1.3.6.1.4.1.21796.4.9.3.1.5';
my $oid_ste2_unit= '.1.3.6.1.4.1.21796.4.9.3.1.7';
my $oid_ste2_sensString= '.1.3.6.1.4.1.21796.4.9.3.1.4';

my $oid_ste_1wireid  = '.1.3.6.1.4.1.21796.4.1.3.1.8';
my $oid_ste_sensName = '.1.3.6.1.4.1.21796.4.1.3.1.2';
my $oid_ste_sensString= '.1.3.6.1.4.1.21796.4.1.3.1.4';
my $oid_ste_sensValue= '.1.3.6.1.4.1.21796.4.1.3.1.5';
my $oid_ste_unit= '.1.3.6.1.4.1.21796.4.1.3.1.7';
my $deviceid = '.1.3.6.1.2.1.1.1.0';
my @oids;
my $response;
my $output;
my $sensor_id;
my $return=0;
my %unit_text  = ( "0"  => "C°","1" => "fahrenheit","2"  => "kelvin","3"  => "%","4"  => "volt","5"=> "miliamper","6"  => "nounit","7"=> "pulse","8"=> "switch"   );
my %unit_ste_text  = ("0"  => "none","1"  => "C°","2" => "fahrenheit","3"  => "kelvin","4"=>"%");


Getopt::Long::Configure('bundling');
GetOptions(
	'h'       => \$opt_help,
	'help'    => \$opt_help,
	'H=s'     => \$opt_host,
	'C=s',    => \$opt_community,
	'S=s',    => \$opt_sensor,
	'I',      => \$opt_contact,
	'w=s'     => \$opt_warning,
	'c=s'     => \$opt_critical,
	'd=s'	  => \$opt_calibre,
	'man'     => \$opt_man,
	'V'	  => \$snmp_version
  );

my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
} 
sub getsnmpOID {
    my($snmp,$oid) = @_;
	my $result = $snmp->get_request("$oid");
	return $$result{"$oid"};
}
sub checkthreshold
{
    # ICI CHECK VALEUR WARNING, CRITICAL (-w -c)
    my ($value,$warning,$critical,$calibre) = @_;
    my $ret = 'OK';
    if($calibre != 0)
    {
        $value += $calibre;
    }
    if($critical =~ m/([\-0-9]*)\,([\-0-9]*)/i) 
    {
        if($value <= $1 || $value >= $2)
        {
            $ret = 'CRITICAL';
        }
    }
    else
    {
        if($value >= $critical)
        {
            $ret = 'CRITICAL';
        }	
    }
    if($ret eq 'OK')
    {
        if($warning =~ m/([\-0-9]*)\,([\-0-9]*)/i) 
        {
            if($value <= $1 || $value >= $2)
            {
                $ret = 'WARNING';
            }
        }
		else
        {
            if($value >= $warning)
            {
                $ret = 'WARNING';
            }
        }
    }
    return $ret;
}
sub sensorArray
{
    my %sensorname;
    my ($snmp, $searchsensor, $oid_wireid, $oid_name) = @_;
    if($searchsensor=~/^([0-9]*)$/)
    {
        # get the sensor ID
        my $result = $snmp->get_table($oid_wireid);	
        foreach my $key ( keys %$result) 
        {
            my $sensorid = $$result{"$key"};
            if($sensorid  =~/$searchsensor/) 
            {
                if($key=~/\.([0-9]*)$/)
                {
                    my $id = $1;
                    $sensorname{$id} = getsnmpOID($snmp,"$oid_name.$id"); 
                }
            }
        }
    }
    else
    {
        # get the sensor ID
        my $result = $snmp->get_table($oid_name);	
        foreach my $key ( keys %$result) 
        {
            my $sensorname = $$result{"$key"};
            if($sensorname =~/$searchsensor/) 
            {
                if($key=~/\.([0-9]*)$/)
                {
                    $sensorname{$1} = $sensorname;
                }
            }
        }
    }
    return (\%sensorname);
}
sub print_help {
	print "Manuel $progname : \n\n";
	print "Ce script a comme objectif de récupérer les données d'un appareil HwG.\n";
	print "Les options disponibles sont :\n";
	print "\t '-C' : (Communauté) spécifie la communauté.\n"; 
	print "\t '-H' : Spécifie l'adresse IP.\n"; 
	print "\t '-S' : (sensor) Spécifie l'identifiant de la sonde.\n";
	print "\t '-I' : Précise si la porte est ouverte ou fermee\n";
	print "\t '-w' : (warning) Spécifie une valeur d'avertissement.\n";
	print "\t '-c' : (critical) Spécifie une valeur critique.\n";
	print "\t '-V' : (version) Spécifie la version SNMP.\n";
	print "\t '--help' : Affiche le manuel du script.\n";
	print "\t '-h' : Affiche le manuel du script.\n\n\n";
}


# Help
if ($opt_help || $opt_man) {
	print_help();
}

# Check if all needed options present.
unless ( $opt_host && ( $opt_sensor || $opt_contact ) && $opt_community ) 
{
	print "\nNot enough options specified !\n";
	print "Display manual with option -h or --help\n";
}
else
{
	# Open SNMP Session
	my ( $snmp, $error ) = Net::SNMP->session(
		-hostname  => $opt_host,
		-community => $opt_community,
		-port      => 161,
		-version   => $snmp_version,		
	);

	# SNMP Session failed
	if ( !defined($snmp) ) {
		print "Unknown error : $error";
		exit(3);
	}

	# Si un ID sensor est bien défini
	if (defined $opt_sensor)
	{
        my $globalval=0;
        my $num=0;
		$response = $snmp->get_request($deviceid);
		my $device = $response->{$deviceid};
		# setting per device OIDs for sensor values
		if ($device =~ m/Poseidon/i) 
		{
            my ($arr) = sensorArray($snmp, $opt_sensor, $oid_poseidon_1wireid, $oid_poseidon_sensName);
            my %arrid = %$arr;
            foreach my $id ( keys %arrid) 
            {
                my $name =$arrid{$id}; 
                my $state="OK";
                my $value = getsnmpOID($snmp,"$oid_poseidon_sensValue.$id");
                if($value==0)
                {
                    
                }
                else
                {
                    $value = $value/10;
                    $value = sprintf( "%.1f", $value);     
                    $state = checkthreshold($value,$opt_warning,$opt_critical,$opt_calibre);  
                }
                my $type="sensors";
                my $unit = getsnmpOID($snmp,"$oid_poseidon_unit.$id");    
                if($unit ==0||$unit ==1)
                { 
                    $type="Temperature of";
                }
                elsif($unit ==3)
                {
                    $type="Humidity sensor";
                }
                print "$state : $type $name is $value ".$unit_text{$unit}."\n";  
                $return = priority_state($return,$ERRORS{$state});
                $globalval=$globalval + $value;
                $num++;
            }
		} 
        else
        {
		    if ($device =~ m/STE2/i) 
		    {
		        my ($arr) = sensorArray($snmp, $opt_sensor, $oid_ste2_1wireid, $oid_ste2_sensName);
                my %arrid = %$arr;
                foreach my $id ( keys %arrid) 
                {
                    my $name =$arrid{$id};      
                    my $value = getsnmpOID($snmp,"$oid_ste2_sensValue.$id");
                    my $state="OK";
                    my $flag=0;
                    if($value==0)
                    {
                        my $str = getsnmpOID($snmp,"$oid_ste2_sensString.$id");
                        if($str eq "")
                        {
                            print "UNKNOWN: $name is error state. check STE web page\n"; 
                            $return = priority_state($return,$ERRORS{UNKNOWN});
                            $flag=1;
                        }
                        else
                        {
                            $state = checkthreshold($value,$opt_warning,$opt_critical,$opt_calibre);  
                        }
                    }
                    else
                    {
                        $value = $value/10;
                        $value = sprintf( "%.1f", $value);      
                    }
                    if($flag == 0)
                    {
                        $state = checkthreshold($value,$opt_warning,$opt_critical,$opt_calibre);  
                        my $unit = getsnmpOID($snmp,"$oid_ste2_unit.$id"); 
                        my $type="sensors";
                        if($unit ==1 || $unit ==2)
                        { 
                            $type="Temperature of";
                        }
                        elsif($unit ==4)
                        {
                            $type="Humidity sensor";
                        }
                        print "$state : $type $name is $value ".$unit_ste_text{$unit}."\n";  
                        $return = priority_state($return,$ERRORS{$state});
                        $globalval=$globalval + $value;
                        $num++;
                    }
                }
		    }
            elsif ($device =~ m/STE/i) 
		    {
		        my ($arr) = sensorArray($snmp,$opt_sensor,$oid_ste_1wireid,$oid_ste_sensName);
                my %arrid = %$arr;
                foreach my $id ( keys %arrid) 
                {
                    my $name =$arrid{$id};      
                    my $value = getsnmpOID($snmp,"$oid_ste_sensValue.$id");
                    my $state="OK";
                    my $flag=0;
                    if($value==0)
                    {
                        my $str = getsnmpOID($snmp,"$oid_ste_sensString.$id");
                        if($str eq "")
                        {
                            print "UNKNOWN: $name is error state. check STE web page\n"; 
                            $return = priority_state($return,$ERRORS{UNKNOWN});
                            $flag=1;
                        }
                        else
                        {
                            $state = checkthreshold($value,$opt_warning,$opt_critical,$opt_calibre);  
                        }
                    }
                    else
                    {
                        $value = $value/10;
                        $value = sprintf( "%.1f", $value);      
                    }
                    if($flag == 0)
                    {
                        $state = checkthreshold($value,$opt_warning,$opt_critical,$opt_calibre);  
                        my $unit = getsnmpOID($snmp,"$oid_ste_unit.$id"); 
                        my $type="sensors";
                        if($unit ==1 || $unit ==2)
                        { 
                            $type="Temperature of";
                        }
                        elsif($unit ==4)
                        {
                            $type="Humidity sensor";
                        }
                        print "$state : $type $name is $value ".$unit_ste_text{$unit}."\n";  
                        $return = priority_state($return,$ERRORS{$state});
                        $globalval=$globalval + $value;
                        $num++;
                    }
                }
		    } 
            else 
            {
			    print "ERROR: device not supported";
			    $snmp->close();
			    exit(3);
		    }
        }
        if($num == 0)
        {
            print "UNKNOWN: no sensors found $opt_sensor\n";
            $return = priority_state($return,$ERRORS{UNKNOWN});
        }   
        else
        {
            $globalval = $globalval/$num;
            $globalval = sprintf( "%.1f", $globalval);
            print "|data=$globalval";
        }
	}

	# Contact
	if (defined $opt_contact) 
	{
		# get the contact values
		#REINITIALISE LE TABLEAU OIDS
		@oids=();
		push(@oids, $enterprise.'.3.3.1.1.3.1');	# POSEIDON-MIB::inpName
		push(@oids, $enterprise.'.3.3.1.1.2.1');	# POSEIDON-MIB::inpValue
		push(@oids, $enterprise.'.3.3.1.1.4.1');	# POSEIDON-MIB::inpAlarmSetup
		push(@oids, $enterprise.'.3.3.1.1.5.1');	# POSEIDON-MIB::inpAlarmState

		# getting the values from the device
		$response = $snmp->get_request(-varbindlist => \@oids) 
                        or print "ERROR: Sensor ID not found\n";

		print $response->{$oids[1]}."\n";

		# setting the output string
		if(($response->{$oids[1]}) == 1)
		{
			$output .= "\nCRITICAL : Porte ouverte";
			$return = 2;
		}		
		else
		{		
			$output .= "\nPorte fermee";
		}
	}
	exit($return);
}

