#!/usr/bin/perl -w 
############################## check_snmp_idrac_syshealth.pl ########
my $Version='1.0';
# Date : Mar 02 2017
# Author  : Faes Nathan
# Help : esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;

my $PROGNAME='check_snmp_idrac_syshealth.pl';
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my $return=0;
my ($opt_h,$opt_V, $opt_G);
my $community = "public";
my $hostname;
my $snmp_version = 2;

my $oid_sysDescr = "1.3.6.1.2.1.1.1";
my $oid_model = "1.3.6.1.4.1.674.10892.5.1.3.12.0";
my $oid_globalSystemStatus = '1.3.6.1.4.1.674.10892.5.2.1.0';

my $oid_eventLogTableEntry = '1.3.6.1.4.1.674.10892.5.4.300.40.1';
my $oid_eventLogRecord = '1.3.6.1.4.1.674.10892.5.4.300.40.1.5';
my $oid_eventLogSeverityStatus = '1.3.6.1.4.1.674.10892.5.4.300.40.1.7';
my $oid_eventLogDateName = '1.3.6.1.4.1.674.10892.5.4.300.40.1.8';

my $oid_fan_fqdd = "1.3.6.1.4.1.674.10892.5.4.700.12.1.8";
my $oid_fan_status = "1.3.6.1.4.1.674.10892.5.4.700.12.1.5";
my $oid_fan_state = "1.3.6.1.4.1.674.10892.5.4.700.12.1.4";
my $oid_fan_speed = "1.3.6.1.4.1.674.10892.5.4.700.12.1.6";
my $oid_cpu_fqdd = "1.3.6.1.4.1.674.10892.5.4.1100.30.1.26";
my $oid_cpu_status = "1.3.6.1.4.1.674.10892.5.4.1100.30.1.5";
my $oid_cpu_state = "1.3.6.1.4.1.674.10892.5.4.1100.30.1.4";
my $oid_cpu_currentspeed = "1.3.6.1.4.1.674.10892.5.4.1100.30.1.12";
my $oid_cpu_corecount = "1.3.6.1.4.1.674.10892.5.4.1100.30.1.17";
my $oid_overall_cpu_status = "1.3.6.1.4.1.674.10892.5.5.1.20.130.4.1.24";
my $oid_power_State = "1.3.6.1.4.1.674.10892.5.2.4.0";

my $PowerStatus = { 1 => 'other',2=>'unknown',3=>'off',4=>'on'};
my $ObjectStatus = { 1 => 'other',2=>'unknown',3=>'ok',4=>'nonCritical (non-critical)',5=>'critical (failure)',6=>'nonRecoverable (dead)'};
my $ObjectStatusToEsia = { 1 => 'OK',2=>'UNKNOWN',3=>'OK',4=>'WARNING',5=>'CRITICAL',6=>'CRITICAL'};

sub print_help() {
	printf "check l'état de santé du système d'un serveur IDRAC Dell.\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)     Hostname to query - (required)\n";
	print " -C (--community)    SNMP read community (defaults to public)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "                     2 for SNMP v2c (default)\n";
	print " -V (--version)      Plugin version\n";
	print " -h (--help)         usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>] [-w warning] [-c critical]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}
my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
}

sub readOID {
    my($snmp,$oid) = @_;
        my $result = $snmp->get_request("$oid");
        return $$result{"$oid"};
}

#
#MAIN
#
&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
    "G"   => \$opt_G,	"global"       => \$opt_G,
	"C=s" => \$community,	"community=s" => \$community,
	"H=s" => \$hostname,	"hostname=s" => \$hostname,
	"v=i" => \$snmp_version,"snmp_version=i" => \$snmp_version,
);
# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }
if (!$hostname) { print_help(); exit $ERRORS{'UNKNOWN'}; }
my ($snmp, $snmperror);
if ($snmp_version == 2) 
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv2c',
		-community => $community,
		-timeout=>2
	);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv1',
		-community => $community,
		-timeout=>2
	);
}
#on interroge en SNMP pour récupérer les informations 
my $result = $snmp->get_request("$oid_sysDescr");

#si l'host ne répond pas on blocke le script
if (!defined($result)) 
{
	if($snmp_version == 2)
	{
		($snmp, $snmperror) = Net::SNMP->session(
			-hostname => $hostname,
			-version => 'snmpv1',
			-community => $community
		);
		$result = $snmp->get_request("$oid_sysDescr");
		if (!defined($result)) 
		{
	   		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
	   		$snmp->close;
	  		 exit $ERRORS{"UNKNOWN"};
		}
	}
	else
	{
		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
		$snmp->close;
		 exit $ERRORS{"UNKNOWN"};
	}
}

my $state = "OK";
my $snmpresult;

if ($opt_G)
{
    my $snmpresult = $snmp->get_request("$oid_globalSystemStatus");
    my $global_status = $$snmpresult{"$oid_globalSystemStatus"};
    my $global_status_str = $ObjectStatus->{"$global_status"};
    my $global_status_esia = $ObjectStatusToEsia->{"$global_status"};
    
    print "$global_status_esia: Device global health status is '$global_status_str'\n";

    if( $global_status > 3 )
    {
        $snmpresult = $snmp->get_table(Baseoid => $oid_eventLogDateName);
        my $str = '';
        
        if(defined( $snmpresult))
        {
            my $list;
            foreach my $key (keys %$snmpresult) 
            {
                if($key =~ /([0-9]*)$/)
                {
                    my $id = $1;
                    $$list{$id} = $$snmpresult{$key};          
                }
            }

            foreach my $id (sort { $$list{$b} cmp $$list{$a} } keys %$list)
            {
                my $severity = readOID($snmp,"$oid_eventLogSeverityStatus.1.$id");
                if( $severity == $global_status )
                {
                    my $severity_str = $global_status_str;
                    my $record = readOID($snmp,"$oid_eventLogRecord.1.$id");
                    my $date = $$list{$id};
                    $date =~ ( /(2\d{3})([0-1]\d)([0-3]\d)([0-2]\d)([0-6]\d)([0-6]\d).*([+-]\d+)/ );
                    $str.= "Last '$severity_str' log is : $record - $4:$5:$6 $3/$2/$1 ($7)\n";
                    last;
                }
            }
        }
        print "$str";
    }
    exit($ERRORS{$global_status_esia});
} 

$snmpresult = $snmp->get_request("$oid_overall_cpu_status");
my $overall_cpu_status =  $$snmpresult{"$oid_overall_cpu_status"};

$snmpresult = $snmp->get_request("$oid_model");
print "Model : ".$$snmpresult{"$oid_model"}."\n";
############## POWER ##############
$snmpresult = $snmp->get_request("$oid_power_State");
my $power_status =  $$snmpresult{"$oid_power_State"};
my $msg_power = 'OK';

if($power_status == 3 )
{
	$msg_power = 'CRITICAL';
	$return = priority_state($return,$ERRORS{CRITICAL});
}
elsif($power_status == 2 || $power_status == 1)
{
	$msg_power = 'UNKNOWN';
	$return = priority_state($return,$ERRORS{UNKNOWN});
}
print "$msg_power: system power state is ".$PowerStatus->{"$power_status"}."\n";


############### FAN ###############

my $fan_fqdd = $snmp->get_table(Baseoid => $oid_fan_fqdd);

if (!defined($fan_fqdd)) 
{
   print "FAN sensors not found\n";
}
else
{
	foreach my $key ( keys %$fan_fqdd) 
	{	
		if($key =~/12\.1\.8\.([0-9]*)\.([0-9]*)/)
		{
			my $id = "$1.$2";
			$snmpresult = $snmp->get_request("$oid_fan_status.$id");
			my $value_status = $$snmpresult{"$oid_fan_status.$id"};
			my $state_status = "OK";
			if($value_status == 5 || $value_status == 8 || $value_status == 10)
			{
				$return = priority_state($return,$ERRORS{CRITICAL});
				if($value_status == 5)
				{
					$state_status = "Critical Upper";
				}
				elsif($value_status == 8)
				{
					$state_status = "Critical Lower";
				}
				else
				{
					$state_status = "Critical";
				}
			}
			elsif($value_status == 4 || $value_status == 6 || $value_status == 7 || $value_status == 9)
			{
				$return = priority_state($return,$ERRORS{WARNING});
				if($value_status == 4)
				{
					$state_status = "Non Critical Upper";
				}
				elsif($value_status == 6)
				{
					$state_status = "Non Recoverable Upper";
				}				
				elsif($value_status == 7)
				{
					$state_status = "Non Critical Lower";
				}
				elsif($value_status == 9)
				{
					$state_status = "Non Recoverable Lower";
				}
				else
				{
					$state_status = "Warning";
				}
			}			
			elsif($value_status == 3)
			{
				$return = priority_state($return,$ERRORS{OK});
				$state_status = "OK";
			}
			else
			{
				$return = priority_state($return,$ERRORS{UNKNOWN});
				$state_status = "UNKNOWN";
			}

			$snmpresult = $snmp->get_request("$oid_fan_state.$id");
			my $value_state = $$snmpresult{"$oid_fan_state.$id"};
			my $state_state = "OK";
			if($value_state == 4)
			{
				$return = priority_state($return,$ERRORS{CRITICAL});
				$state_state = "Not Ready";
			}
			elsif($value_state == 6)
			{
				$return = priority_state($return,$ERRORS{CRITICAL});
				$state_state = "Enabled and not ready";
			}
			elsif($value_state == 2)
			{
				$return = priority_state($return,$ERRORS{OK});
				$state_state = "Enabled";
			}
			else
			{
				$return = priority_state($return,$ERRORS{UNKNOWN});
				$state_state = "Unknown";
			}

			
			$snmpresult = $snmp->get_request("$oid_fan_speed.$id");
			print "$$fan_fqdd{$key} : $state_status ; $state_state ; Speed : ".$$snmpresult{"$oid_fan_speed.$id"}." RPM\n";	

		}
	}
}

############### CPU ###############

my $cpu_fqdd = $snmp->get_table(Baseoid => $oid_cpu_fqdd);

if (!defined($cpu_fqdd)) 
{
   print "FQDD CPU na\n";
}
else
{
	foreach my $key ( keys %$cpu_fqdd) 
	{	
		if($key =~/30\.1\.26\.([0-9]*)\.([0-9]*)/)
		{
			my $id = "$1.$2";
			$snmpresult = $snmp->get_request("$oid_cpu_status.$id");
			my $value_status = $$snmpresult{"$oid_cpu_status.$id"};
			my $state_status = "OK";
			if($value_status == 5 || $value_status == 8 || $value_status == 10)
			{
				$return = priority_state($return,$ERRORS{CRITICAL});
				if($value_status == 5)
				{
					$state_status = "Critical Upper";
				}
				elsif($value_status == 8)
				{
					$state_status = "Critical Lower";
				}
				else
				{
					$state_status = "Critical";
				}
			}
			elsif($value_status == 4 || $value_status == 6 || $value_status == 7 || $value_status == 9)
			{
				$return = priority_state($return,$ERRORS{WARNING});
				if($value_status == 4)
				{
					$state_status = "Non Critical Upper";
				}
				elsif($value_status == 6)
				{
					$state_status = "Non Recoverable Upper";
				}				
				elsif($value_status == 7)
				{
					$state_status = "Non Critical Lower";
				}
				elsif($value_status == 9)
				{
					$state_status = "Non Recoverable Lower";
				}
				else
				{
					$state_status = "Warning";
				}
			}			
			elsif($value_status == 3)
			{
				$return = priority_state($return,$ERRORS{OK});
				$state_status = "OK";
			}
			else
			{
				$return = priority_state($return,$ERRORS{UNKNOWN});
				$state_status = "UNKNOWN";
			}

			$snmpresult = $snmp->get_request("$oid_cpu_state.$id");
			my $value_state = $$snmpresult{"$oid_cpu_state.$id"};
			my $state_state = "OK";
			if($value_state == 4)
			{
				$return = priority_state($return,$ERRORS{CRITICAL});
				$state_state = "Not Ready";
			}
			elsif($value_state == 6)
			{
				$return = priority_state($return,$ERRORS{CRITICAL});
				$state_state = "Enabled and not ready";
			}
			elsif($value_state == 2)
			{
				$return = priority_state($return,$ERRORS{OK});
				$state_state = "Enabled";
			}
			else
			{
				$return = priority_state($return,$ERRORS{UNKNOWN});
				$state_state = "Unknown";
			}

			
			$snmpresult = $snmp->get_request("$oid_cpu_currentspeed.$id");
			my $current_speed = $$snmpresult{"$oid_cpu_currentspeed.$id"};

			$snmpresult = $snmp->get_request("$oid_cpu_corecount.$id");
			my $core_count = $$snmpresult{"$oid_cpu_corecount.$id"};

			print "$$cpu_fqdd{$key} : $state_status ; $state_state ; Current Speed : ".$current_speed." MHz ; Core count : $core_count\n";
		}
	}
}
exit($return);

